@@ -1,3 +1,4 @@
+# pylint:disable=global-statement,missing-class-docstring,no-self-use,unspecified-encoding
 import os
 import sys
 import subprocess
@@ -5,7 +6,8 @@ import logging
 from pathlib import Path
 from typing import Tuple, Optional
 
-import toml
+import tomlkit
+import tomlkit.exceptions
 from xdg import BaseDirectory
 from PySide2.QtWidgets import QApplication
 from PySide2.QtWidgets import QMessageBox, QFileDialog
@@ -18,6 +20,7 @@ _l = logging.getLogger(name=__name__)
 
 # we probably want to put this feature into angr management
 _app = None
+
 def tmp_app():
     global _app
     if _app is None:
@@ -87,15 +90,16 @@ class ChessUrlHandler(BasePlugin):
             return rootdirs_path
         raise ValueError("Cannot get the configuration file root directory for angr-management.")
 
-    def _get_rootdir(self, target_uuid: str, challenge_name: str, source_file: str) -> Tuple[Optional[str],Optional[str]]:
+    def _get_rootdir(self, target_uuid: str, challenge_name: str,
+                     source_file: str) -> Tuple[Optional[str],Optional[str]]:
         rootdirs_path = self._get_rootdir_config_path()
         # load it if it exists
         entries = { }
         if os.path.isfile(rootdirs_path):
             with open(rootdirs_path, "r") as f:
                 try:
-                    entries = toml.load(f)
-                except toml.TomlDecodeError:
+                    entries = tomlkit.load(f)
+                except tomlkit.exceptions.ParseError:
                     _l.error("Cannot decode rootdirs file %s. Ignore existing content.",
                              rootdirs_path)
 
@@ -103,7 +107,7 @@ class ChessUrlHandler(BasePlugin):
         if 'uuid_to_rootdir' in entries:
             if target_uuid in entries['uuid_to_rootdir']:
                 # yes there is already one
-                dir_path = entries['uuid_to_rootdir'][target_uuid]
+                dir_path = entries['uuid_to_rootdir'][target_uuid].value
             else:
                 dir_path = None
 
@@ -118,7 +122,7 @@ class ChessUrlHandler(BasePlugin):
                 full_path = None
 
                 source_file_name = os.path.basename(source_file)
-                for base, directories, files in os.walk(dir_path):
+                for base, _, files in os.walk(dir_path):
                     if source_file_name in files:
                         # check the full path
                         full_path = os.path.normpath(os.path.join(base, source_file_name))
@@ -161,8 +165,8 @@ class ChessUrlHandler(BasePlugin):
         if os.path.isfile(rootdirs_path):
             with open(rootdirs_path, "r") as f:
                 try:
-                    entries = toml.load(f)
-                except toml.TomlDecodeError:
+                    entries = tomlkit.load(f)
+                except tomlkit.exceptions.ParseError:
                     _l.error("Cannot decode rootdirs file %s. Ignore existing content.",
                              rootdirs_path)
 
@@ -175,7 +179,7 @@ class ChessUrlHandler(BasePlugin):
 
         # store it
         with open(rootdirs_path, "w") as f:
-            toml.dump(entries, f)
+            tomlkit.dump(entries, f)
 
     def _vscode_path(self) -> Optional[str]:
         if sys.platform == "win32":
